#ifdef WIN32
#include <windows.h>
#endif

#include <ode/ode.h>
#include <drawstuff/drawstuff.h>

#ifdef dDOUBLE
  #define dsDrawBox dsDrawBoxD
  #define dsDrawSphere dsDrawSphereD
  #define dsDrawCylinder dsDrawCylinderD
#endif

static dWorldID world;
static dBodyID body_box;
static dBodyID body_table;
static dBodyID body_gyroscope;
dJointID joint_hinge1;
dJointID joint_hinge2;

dReal box_size[3] = { 1.0, 1.0, 1.0 };
dReal table_radius = 3.0;
dReal table_height = 0.5;
dReal gyroscope_radius = 1.0;
dReal gyroscope_height = 0.5;

#define STEP_SIZE 0.01
double time = 0;

 
// start simulation - set viewpoint
static void start()
{
    static float xyz[3] = { 0.f, -10.f, 5.f };
    static float hpr[3] = { 90.f, -15.f, 0.f };

    dsSetViewpoint( xyz, hpr );
}

// called when a key pressed
static void command (int cmd)
{
    if (cmd == 'a') {
        dJointAddHingeTorque( joint_hinge2, 10.0 );
        printf( "torque = %f\n", 10.0 );
    }
    else if (cmd == 'd') {
        dJointAddHingeTorque( joint_hinge2, -10.0 );
        printf( "torque = %f\n", -10.0 );
    }
}
 
// simulation loop
static void simLoop( int pause )
{
    // Ctl+p ꂽifɓȂ
    if (!pause)
    {
        dWorldStep( world, STEP_SIZE );
        time += STEP_SIZE;
    }
 
    dsSetColor( 1.0f, 1.0f, 1.0f );
    dsDrawBox( dBodyGetPosition( body_box ),
        dBodyGetRotation( body_box ), box_size );

    dsSetColor( 1.0f, 1.0f, 0.0f );
    dsDrawCylinder( dBodyGetPosition( body_table ),
        dBodyGetRotation( body_table ), table_height, table_radius );

    dsSetColor( 0.0f, 1.0f, 0.0f );
    dsDrawCylinder( dBodyGetPosition( body_gyroscope ),
        dBodyGetRotation( body_gyroscope ), gyroscope_height, gyroscope_radius );
}
 
 
int main( int argc, char* argv[] )
{
    dInitODE();
 
    // setup pointers to drawstuff callback functions
    dsFunctions fn;
    fn.version = DS_VERSION;
    fn.start   = &start;
    fn.step    = &simLoop;
    fn.command = &command;
    fn.stop    = 0;
    fn.path_to_textures = "../drawstuff/textures";
 
    world = dWorldCreate();
    dWorldSetGravity( world, 0.0, 0.0, -9.8 );

 
    // box creating
    {
        body_box = dBodyCreate( world );
        dBodySetPosition( body_box, 0.0, 0.0, 0.5 );
    }
    // box̌Œ
    {
        dJointID fixed;
        fixed = dJointCreateFixed( world, 0 );
        dJointAttach( fixed, NULL, body_box );
        dJointSetFixed( fixed );
    }

    // turn table creating
    {
        body_table = dBodyCreate( world );
        dBodySetPosition( body_table, 0.0, 0.0, 1.0 );
        
        dMass m;
        dMassSetCylinderTotal( &m, 10.0, 1, table_radius, table_height );
        dBodySetMass( body_table, &m );

    }
    // qWWCg(turn table)
    {
        joint_hinge1 = dJointCreateHinge( world, 0 );
        dJointAttach( joint_hinge1, body_box, body_table );
        dJointSetHingeAnchor( joint_hinge1, 0.0, 0.0, 1.0 );
        dJointSetHingeAxis( joint_hinge1, 0.0, 0.0, 1.0 );
    }

    // gyroscope creating
    {
        body_gyroscope = dBodyCreate( world );
        dBodySetPosition( body_gyroscope, 2.0, 0.0, 5.0 );
        
        dMass m;
        dMassSetCylinderTotal( &m, 10.0, 1, gyroscope_radius, gyroscope_height );
        dBodySetMass( body_gyroscope, &m );

        dMatrix3 R;
        dRFromAxisAndAngle( R, 1.0, 0.0, 0.0, 60.0*M_PI/180.0 );
        dBodySetRotation( body_gyroscope, R );
    }
    // qWWCg(gyroscope)
    {
        joint_hinge2 = dJointCreateHinge( world, 0 );
        dJointAttach( joint_hinge2, body_table, body_gyroscope );
        dJointSetHingeAnchor( joint_hinge2, 2.0, 0.0, 5.0 ); // gyroscopeƓW
        dJointSetHingeAxis( joint_hinge2, 0.0, -1.73, 1.0 );
    }


    dsSimulationLoop( argc, argv, 320, 240, &fn );

    dWorldDestroy( world );
    dCloseODE();
    return 0;
}



